import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

const EdgeInsets _defaultInsetPadding = EdgeInsets.symmetric(horizontal: 0.0, vertical: 0.0);

const String TAG = "InputDialog";

class InputDialog extends StatefulWidget {

  const InputDialog({
    Key key,
    this.backgroundColor,
    this.elevation,
    this.insetAnimationDuration = const Duration(milliseconds: 100),
    this.insetAnimationCurve = Curves.decelerate,
    this.insetPadding = _defaultInsetPadding,
    this.clipBehavior = Clip.none,
    this.shape,
    this.child,
    this.onSubmit,
    this.textController,
  }) : assert(clipBehavior != null),
        super(key: key);

  final Color backgroundColor;

  /// {@template flutter.material.dialog.elevation}
  /// The z-coordinate of this [Dialog].
  ///
  /// If null then [DialogTheme.elevation] is used, and if that's null then the
  /// dialog's elevation is 24.0.
  /// {@endtemplate}
  /// {@macro flutter.material.material.elevation}
  final double elevation;

  /// {@template flutter.material.dialog.insetAnimationDuration}
  /// The duration of the animation to show when the system keyboard intrudes
  /// into the space that the dialog is placed in.
  ///
  /// Defaults to 100 milliseconds.
  /// {@endtemplate}
  final Duration insetAnimationDuration;

  /// {@template flutter.material.dialog.insetAnimationCurve}
  /// The curve to use for the animation shown when the system keyboard intrudes
  /// into the space that the dialog is placed in.
  ///
  /// Defaults to [Curves.decelerate].
  /// {@endtemplate}
  final Curve insetAnimationCurve;

  /// {@template flutter.material.dialog.insetPadding}
  /// The amount of padding added to [MediaQueryData.viewInsets] on the outside
  /// of the dialog. This defines the minimum space between the screen's edges
  /// and the dialog.
  ///
  /// Defaults to `EdgeInsets.symmetric(horizontal: 40.0, vertical: 24.0)`.
  /// {@endtemplate}
  final EdgeInsets insetPadding;

  /// {@template flutter.material.dialog.clipBehavior}
  /// Controls how the contents of the dialog are clipped (or not) to the given
  /// [shape].
  ///
  /// See the enum [Clip] for details of all possible options and their common
  /// use cases.
  ///
  /// Defaults to [Clip.none], and must not be null.
  /// {@endtemplate}
  final Clip clipBehavior;

  /// {@template flutter.material.dialog.shape}
  /// The shape of this dialog's border.
  ///
  /// Defines the dialog's [Material.shape].
  ///
  /// The default shape is a [RoundedRectangleBorder] with a radius of 4.0
  /// {@endtemplate}
  final ShapeBorder shape;

  /// The widget below this widget in the tree.
  ///
  /// {@macro flutter.widgets.child}
  final Widget child;

  final void Function(String value) onSubmit;
  
  final TextEditingController textController;
  
  @override
  _InputDialogState createState() => _InputDialogState();
}

class _InputDialogState extends State<InputDialog> {
  static const RoundedRectangleBorder _defaultDialogShape =
  RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(4.0)));

  static const double _defaultElevation = 24.0;

  @override
  Widget build(BuildContext context) {
    final DialogTheme dialogTheme = DialogTheme.of(context);
    final EdgeInsets effectivePadding = MediaQuery.of(context).viewInsets + (widget.insetPadding ?? const EdgeInsets.all(0.0));
    TextField _textField = TextField(
      controller: widget.textController,
      autofocus: true,
      keyboardType: TextInputType.multiline,
      maxLines: 2,
      minLines: 1,
      cursorColor: Color(0xFFFF0000),
      decoration: InputDecoration(
          enabledBorder: OutlineInputBorder(
              borderSide: BorderSide(
                  color: Color(0xFFFF0000)
              )
          ),
          focusedBorder: OutlineInputBorder(
              borderSide: BorderSide(
                  color: Color(0xFFFF0000)
              )
          ),
          contentPadding: const EdgeInsets.symmetric(horizontal: 10, vertical: 5)
      ),
      style: TextStyle(color: Color(0xFFFF0000), fontSize: 18),
    );
    return AnimatedPadding(
      padding: effectivePadding,
      duration: widget.insetAnimationDuration,
      curve: widget.insetAnimationCurve,
      child: MediaQuery.removeViewInsets(
        removeLeft: true,
        removeTop: true,
        removeRight: true,
        removeBottom: true,
        context: context,
        child: Center(
          child: ConstrainedBox(
            constraints: const BoxConstraints(minWidth: 280.0),
            child: Material(
              color: widget.backgroundColor ?? dialogTheme.backgroundColor ?? Theme.of(context).dialogBackgroundColor,
              elevation: widget.elevation ?? dialogTheme.elevation ?? _defaultElevation,
              shape: widget.shape ?? dialogTheme.shape ?? _defaultDialogShape,
              type: MaterialType.card,
              clipBehavior: widget.clipBehavior,
              child: GestureDetector(
                behavior: HitTestBehavior.opaque,
                child: Center(
                  widthFactor: double.infinity,
                  heightFactor: double.infinity,
                  child: Container(
                    width: 300,
                    child: _textField,
                  ),
                ),
                onTap: onBackGroundTap,
              )
            ),
          ),
        ),
      ),
    );
  }

  void onBackGroundTap() {
    print("$TAG onTap");
    FocusNode focusNode = FocusScope.of(context);
    if (focusNode.hasFocus) {
      focusNode.unfocus();
    }
    widget.onSubmit(widget.textController.text);
    Navigator.of(context).pop();
  }
}