import 'package:tiw_flutter/teduboard_controller.dart';

import 'teduboard_def.dart';

/// 白板通用事件回调
abstract class TeduBoardCommonCallbacksHandler {

  /// 白板错误回调
  /// @param code 错误码
  /// @param msg 错误信息
  void onTEBError(TEduBoardErrorCode code, String msg) {}

  /// 白板警告回调
  /// @param code 错误码
  /// @param msg  错误信息
  void onTEBWarning(TEduBoardWarningCode code, String msg) {}

}

/// 白板基本流程回调 
abstract class TeduBoardProcessCallbacksHandler {

  /// 白板初始化完成回调
  /// 收到该回调后表示白板已处于可正常工作状态（此时白板为空白白板，历史数据尚未拉取到）  
  void onTEBInit(TEduBoardController boardController) {}

  /// 白板历史数据同步完成回调
  void onTEBHistroyDataSyncCompleted() {}

  /// 白板同步数据回调
  /// @param data 白板同步数据（JSON 格式字符串）
  /// 收到该回调时需要将回调数据通过信令通道发送给房间内其他人，接受者收到后调用 addSyncData 接口将数据添加到白板以实现数据同步
  /// 该回调用于多个白板间的数据同步
  void onTEBSyncData(String data) {}

  /// 白板可撤销状态改变回调
  /// @param canUndo 白板当前是否还能执行 Undo 操作
  void onTEBUndoStatusChanged(bool canUndo) {}

  /// 白板可重做状态改变回调
  /// @param canRedo 白板当前是否还能执行 Redo 操作
  
  void onTEBRedoStatusChanged(bool canRedo) {}

}

/// 白板涂鸦功能回调
abstract class TeduBoardGraffitiCallbacksHandler {

  /// 白板状态改变回调
  /// @param boardId 白板 ID
  /// @param url 白板图片 URL
  /// @param status 新的白板图片状态
  void onTEBImageStatusChanged(String boardId, String url, TEduBoardImageStatus status) {}

  ///  设置白板背景图片回调
  /// @param url 调用 SetBackgroundImage 时传入的 URL
  void onTEBSetBackgroundImage(String url) {}

  /// 添加元素回调
  /// @param elementId 调用 addElement 时返回的元素 ID
  /// @param url 调用 addElement 时传入的 url
  void onTEBAddElement(String elementId, String url) {}

  /// 删除元素回调
  /// @param elementIds 删除的元素 IDs
  void onTEBRemoveElement(List elementIds) {}

  /// 设置白板背景 H5 状态改变回调
  /// @param boardId 白板 ID
  /// @param url 白板图片 URL
  /// @param status 新的白板图片状态
  void onTEBBackgroundH5StatusChanged(String boardId, String url, TEduBoardBackgroundH5Status status) {}

}

/// 白板操作回调
abstract class TeduBoardOperationCallbacksHandler {

  /// 增加白板回调
  /// @param boardIds 增加的白板页 ID 列表
  /// @param fileId 增加的白板页所属的文件 ID（目前版本只可能为#DEFAULT）
  void onTEBAddBoard(List boardIds, String fileId) {}

  /// 删除白板回调
  /// @param boardIds 删除的白板页 ID 列表
  /// @param fileId 增加的白板页所属的文件 ID（目前版本只可能为#DEFAULT）
  void onTEBDeleteBoard(List boardIds, String fileId) {}

  /// 跳转白板回调
  /// @param boardId 跳转到的白板页 ID
  /// @param fileId 跳转到的白板页所属的文件 ID
  void onTEBGotoBoard(String boardId, String fileId) {}

  /// 白板页动画步数回调
  /// @param currentStep 当前白板页动画步数，取值范围 [0, totalStep)
  /// @param totalStep 当前白板页动画总步数
  void onTEBGotoStep(int currentStep, int totalStep) {}

  /// 框选工具选中回调
  /// 只有框选中涂鸦或图片元素后触发回调
  void onTEBRectSelected() {}

  /// 刷新白板回调
  void onTEBRefresh() {}

  /// 白板快照
  /// @param path 快照本地路径
  /// @param code 错误码，返回 0，表示获取快照成功
  /// @param msg 错误信息
  void onTEBSnapshot(String path, TEduBoardErrorCode code, String msg) {}

}

/// 文件操作回调
abstract class TeduBoardFileOperationCallbacksHandler {

  /// 文件转码进度回调
  /// @param result 文件转码结果
  /// @param path 正在转码的本地文件路径
  /// @param errorCode 文件转码错误码，无异常时为空字符串 ""
  /// @param errorMsg 文件转码错误信息，无异常时为空字符串 ""
  void onTEBFileTranscodeProgress(TEduBoardTranscodeFileResult result, String path, String errorCode, String errorMsg) {}

  /// 增加转码文件回，文件加载完成后才会触发该回调
  /// @param fileId 增加的文件 ID
  void onTEBAddTranscodeFile(String fileId) {}

  /// 删除文件回调
  /// @param fileId 删除的文件 ID
  void onTEBDeleteFile(String fileId) {}

  /// 切换文件回调
  /// @param fileId 切换到的文件 ID
  void onTEBSwitchFile(String fileId) {}

  /// 文件上传进度回调
  /// @param path 正在上传的文件路径
  /// @param currentBytes 当前已上传大小，单位 bytes
  /// @param totalBytes 文件总大小，单位 bytes
  /// @param uploadSpeed 文件上传速度，单位 bytes
  /// @param persent 文件上传进度，取值范围 [0, 1]
  void onTEBFileUploadProgress(String path, int currentBytes, int totalBytes, num uploadSpeed, num percent) {}

  /// 文件上传状态回调
  /// @param path 正在上传的文件路径
  /// @param status 文件上传状态
  /// @param errorCode 文件上传错误码
  /// @param errorMsg 文件上传错误信息
  void onTEBFileUploadStatus(String path, TEduBoardUploadStatus status, int errorCode, String errorMsg) {}

  /// H5 文件状态回调
  /// @param fileId 文件 ID
  /// @param statue 文件状态
  void onTEBH5FileStatusChanged(String fileId, TEduBoardH5FileStatus status) {}

  /// 视频文件状态回调
  /// @param fileId 文件 ID
  /// @param status 文件状态
  /// @param progress 当前进度（秒）（仅支持 mp4 格式）
  /// @param duration 总时长（秒）（仅支持 mp4 格式）
  void onTEBVideoStatusChanged(String fileId, TEduBoardVideoStatus status, num progress, num duration) {}

  /// 音频文件状态回调
  /// @param elementId 元素 ID
  /// @param status 文件状态
  /// @param progress 当前进度（秒）
  /// @param duration 总时长
  void onTEBAudioStatusChanged(String elementId, TEduBoardAudioStatus status, num progress, num duration) {}

  /// 增加批量图片文件回调
  /// @param fileId 增加的文件 ID
  void onTEBAddImagesFile(String fileId) {}

  /// H5PPT 文件状态改变回调
  /// @param fileId 文件 ID
  /// @param status 文件状态
  /// @param message 状态消息
  void onTEBH5PPTStatusChanged(String fileId, TEduBoardH5PPTStatus status, String message) {}

}
