import 'dart:convert';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:tiw_flutter/tiw_flutter.dart';
import 'package:tiw_flutter/tiw_log.dart';

import 'teduboard_def.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'utils.dart';
import 'package:string_validator/string_validator.dart';

const String kTEduBoardVersion = '2.5.7';

class TEduBoardController {
  /// 白板
  final TeduBoardView _boardView;
  final WebViewController _webViewController;

  /// sessionId
  final String _sessionId = '12345';

  /// Random
  final String _globalRandom = '12345';

  TEduBoardController(
    this._boardView,
    this._webViewController,
  );

  ///执行js脚本
  Future<String> _evaluateJavaScript(String jsCode) async {
    String result = await _webViewController.evaluateJavascript(jsCode);
    TIWLog.logEvent('run script: $jsCode \n result: ${result ?? ''}');
    return result;
  }

  /// evaluateJavaScript时，如果返回值是字符串，Android这边收到的字符串前后有引号
  /// 使用此方法可获取不带引号的值
  Future<String> _jsonDecodeEvaluateJavaScript(String jsCode) async {
    String result = await _evaluateJavaScript(jsCode);
    if (defaultTargetPlatform == TargetPlatform.iOS) {
      return result;
    }
    return jsonDecode(result);
  }

  ///基本流程
  /**
   * @brief 初始化白板
   * @param authParam         授权参数
   * @param roomId             课堂 ID
   * @param initParam              可选参数，指定用于初始化白板的一系列属性值
   * @warning 使用腾讯云 IMSDK 进行实时数据同步时，只支持一个白板实例，创建多个白板实例可能导致涂鸦状态异常
   */
  void initBoard() {
    TEduBoardAuthParam authParam = _boardView.authParam;
    int roomId = _boardView.roomId;

    TEduBoardInitParam initParam = _boardView.initParam;
    initParam ??= TEduBoardInitParam();

    String json = """{
        id:'board',
        classId:$roomId,
        boardMode:1,
        sdkAppId:${authParam.sdkAppId},
        userId:'${authParam.userId}',
        userSig:'${authParam.userSig}',
        nativeVersion:'$kTEduBoardVersion',
        platform:'${Platform.isIOS ? 'iOS' : 'android'}',
        ratio:'${initParam.ratio}',
        drawEnable:${initParam.drawEnable},
        textStyle:${initParam.textStyle.index},
        textSize:${initParam.textSize},
        textColor:'${rgbColor(initParam.textColor)}',
        brushColor:'${rgbColor(initParam.brushColor)}',
        brushThin:${initParam.brushThin},
        toolType:${initParam.toolType.index},
        globalBackgroundColor:'${rgbColor(initParam.globalBackgroundColor)}',
        boardContentFitMode:${initParam.boardContentFitMode.index},
        dataSyncEnable:${initParam.dataSyncEnable},
        preloadDepth:${initParam.preloadDepth},
        smoothLevel:${initParam.smoothLevel},
        imageTimeout:${initParam.imageTimeout},
        progressEnable:${initParam.progressEnable},
        progressBarUrl:'${initParam.progressBarUrl}',
        systemCursorEnable:${initParam.systemCursorEnable},
        enableScaleTool:${initParam.enableScaleTool},
        sessionId:$_sessionId,
        globalRandom:$_globalRandom,
      }""";
    _evaluateJavaScript("initBoard($json);");
  }

  /**
  * @brief 反初始化白板
  * @warning 调用反初始化接口后会释放内部资源，白板功能将失效。
  */
  void unInit() {
    _evaluateJavaScript("this.board.destroy();");
  }

  ///
  ///@brief 重置白板
  ///调用该接口后将会删除所有的白板页和文件
  ///
  void reset() {
    _evaluateJavaScript("this.board.reset();");
  }

  ///
  ///@brief 获取版本号
  ///@return NSString  版本号字符串
  ///
  String getVersion() {
    return kTEduBoardVersion;
  }

  ///数据同步
/**
 * @brief 添加白板同步数据
 * @param data                 接收到的房间内其他人发送的同步数据
 *
 * 该接口用于多个白板间的数据同步，使用内置 IM 作为信令通道时，不需要调用该接口
*/
  void addSyncData(String data) {
    if (data == null) {
      return;
    }
    _evaluateJavaScript("this.board.addSyncData(JSON.parse('$data'));");
  }

/**
 * @brief 设置白板是否开启数据同步
 * @param enable    是否开启
 *
 * 白板创建后默认开启数据同步，关闭数据同步，本地的所有白板操作不会同步到远端和服务器
*/
  void setDataSyncEnable(bool enable) {
    _evaluateJavaScript("this.board.setDataSyncEnable($enable);");
  }

/**
 * @brief 获取白板是否开启数据同步
 * @return 是否开启数据同步，true 表示开启，false 表示关闭
*/
  Future<bool> isDataSyncEnable() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.isDataSyncEnable();");
    return toBoolean(result);
  }

  ///sync time
/**
 * @brief 获取同步时间戳
 * @return 毫秒级同步时间戳
*/
  Future<int> getSyncTime() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getSyncTime();");
    return int.parse(result);
  }

/**
 * @brief 同步远端时间戳
 * @param userId                远端用户 ID
 * @param timestamp         远端用户毫秒级同步时间戳
*/
  void syncRemoteTime(String userId, int timestamp) {
    _evaluateJavaScript("this.board.syncRemoteTime('$userId',$timestamp);");
  }

/**
 * @brief 添加资源主备域名映射
 * 主备域名均需包含协议类型(支持http/https)
 * 切换域名重试超时默认为5s
 * 多次调用此接口，可为统一主域名添加多个备用域名。如果备用域名已在列表中则不再添加
 * 当主域名不可用时，SDK将按从前往后的顺序从列表中选择一个备用域名并重试，请务必将可用性高的域名添加到列表前面
 *
 * @param domain 主域名 (必填）
 * @param backup 备用域名 (必填)
 * @param priority 优先级，备用域名优先级相同时，按添加的顺序决定优先级，添加越前优先级越高
 */
  void addBackupDomain(String domain, String backup, int priority) {
    _evaluateJavaScript(
        "this.board.addBackupDomain('$domain', '$backup', $priority);");
  }

/**
 * @brief 删除资源主备域名映射
 * @param domain 主域名 (必填）
 * @param backup 备用域名 (必填)
*/
  void removeBackupDomain(String domain, String backup) {
    _evaluateJavaScript("this.board.addBackupDomain('$domain', '$backup');");
  }

  ///涂鸦相关接口

/**
 * @brief 设置允许操作哪些用户绘制的图形
 * @param users             指定允许操作的用户集，为 nullptr 表示不加限制
 *
 * 该接口会产生以下影响：
 *    1. ERASER 工具只能擦除 users 参数列出的用户绘制的涂鸦，无法擦除其他人绘制的涂鸦
 *    2. POINTSELECT、SELECT 工具只能选中 users 参数列出的用户绘制的涂鸦，无法选中其他人绘制的涂鸦
 *    3. clear 接口只能用于清空选中涂鸦以及 users 参数列出的用户绘制的涂鸦，无法清空背景及其他人绘制的涂鸦
 *    4. 白板包含的其他功能未在本列表明确列出者都可以确定不受本接口影响
*/
  void setAccessibleUsers(List<String> users) {
    String params = "[";
    for (var user in users) {
      params += "'$user',";
    }
    params.replaceRange(params.length - 1, null, ']');
    _evaluateJavaScript("this.board.setAccessibleUsers($params);");
  }

/**
 * @brief 设置白板是否允许涂鸦
 * @param enable             是否允许涂鸦，true 表示白板可以涂鸦，false 表示白板不能涂鸦
 *
 * 白板创建后默认为允许涂鸦状态
*/
  void setDrawEnable(bool enable) {
    _evaluateJavaScript("this.board.setDrawEnable($enable);");
  }

/**
 * @brief 获取白板是否允许涂鸦
 * @return 是否允许涂鸦，true 表示白板可以涂鸦，false 表示白板不能涂鸦
*/
  Future<bool> isDrawEnable() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.isDrawEnable();");
    return toBoolean(result);
  }

/**
 * @brief 设置所有白板的背景色
 * @param color             要设置的全局背景色
 *
 * 调用该接口将导致所有白板的背景色发生改变
 * 新创建白板的默认背景色取全局背景色
*/
  void setGlobalBackgroundColor(int hex) {
    _evaluateJavaScript(
        "this.board.setGlobalBackgroundColor('${rgbColor(hex)}');");
  }

/**
 * @brief 获取白板全局背景色
 * @return 全局背景色
*/
  Future<int> getGlobalBackgroundColor() async {
    String result =
        await _jsonDecodeEvaluateJavaScript("this.board.getGlobalBackgroundColor();");
    return hexColor(result);
  }

/**
 * @brief 设置当前白板页的背景色
 * @param color             要设置的背景色
 * 
 * 白板页创建以后的默认背景色由 SetDefaultBackgroundColor 接口设定
*/
  void setBackgroundColor(int hex) {
    _evaluateJavaScript("this.board.setBackgroundColor('${rgbColor(hex)}');");
  }

/**
 * @brief 获取当前白板页的背景色
 * @return 当前白板页的背景色
*/
  Future<int> getBackgroundColor() async {
    String result =
        await _jsonDecodeEvaluateJavaScript("this.board.getBackgroundColor();");
    return hexColor(result);
  }

/**
 * @brief 设置当前白板页的背景图片
 * @param url                 要设置的背景图片 URL，编码格式为 UTF8
 * @param mode                要使用的图片填充对齐模式
 *
 * 当URL是一个有效的本地文件地址时，该文件会被自动上传到 COS
*/
  Future<void> setBackgroundImage(
      String url, TEduBoardImageFitMode mode) async {
    String data = await fileData(url);

    if (data == null || data.length <= 0) {
      _evaluateJavaScript(
          "this.board.setBackgroundImage('$url', ${mode.index});");
      return;
    }

    //本地文件
    _evaluateJavaScript("this.board.setBackgroundImage($data, ${mode.index});");
    return;
  }

/**
 * @brief 设置当前白板页的背景 H5 页面
 * @param url                要设置的背景 H5 页面 URL
 *
 * 该接口与 SetBackgroundImage 接口互斥
*/
  void setBackgroundH5(String url) {
    _evaluateJavaScript("this.board.setBackgroundH5('$url');");
  }

/**
 * @brief 设置要使用的白板工具
 * @param type                 要设置的白板工具
*/
  void setToolType(TEduBoardToolType type) {
    _evaluateJavaScript("this.board.setToolType(${type.index});");
  }

/**
 * @brief 获取正在使用的白板工具
 * @return 正在使用的白板工具
*/
  Future<TEduBoardToolType> getToolType() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getToolType();");
    if (result == null) {
      return null;
    }
    return TEduBoardToolType.values[int.parse(result)];
  }

/**
 * @brief 设置画笔颜色
 * @param color             要设置的画笔颜色
 *
 * 画笔颜色用于所有涂鸦绘制
*/
  void setBrushColor(int hex) {
    _evaluateJavaScript("this.board.setBrushColor('${rgbColor(hex)}');");
  }

/**
 * @brief 获取画笔颜色
 * @return 画笔颜色
*/
  Future<int> getBrushColor() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getBrushColor();");
    return hexColor(result);
  }

/**
 * @brief 设置画笔粗细
 * @param thin                 要设置的画笔粗细
 *
 * 画笔粗细用于所有涂鸦绘制，实际像素值取值(thin * 白板的高度 / 10000)px，如果结果小于1px，则涂鸦的线条会比较虚
*/
  void setBrushThin(int thin) {
    _evaluateJavaScript("this.board.setBrushThin($thin);");
  }

/**
 * @brief 获取画笔粗细
 * @return 画笔粗细
*/
  Future<int> getBrushThin() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getToolType();");
    if (result == null) {
      return null;
    }
    return int.parse(result);
  }

/**
 * @brief 设置文本颜色
 * @param color             要设置的文本颜色
*/
  void setTextColor(int hex) {
    _evaluateJavaScript("this.board.setTextColor('${rgbColor(hex)}');");
  }

/**
 * @brief 获取文本颜色
 * @return 文本颜色
*/
  Future<int> getTextColor() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getTextColor();");
    return hexColor(result);
  }

/**
 * @brief 设置文本样式
 * @param style             要设置的文本样式
*/
  void setTextStyle(TEduBoardTextStyle style) {
    _evaluateJavaScript("this.board.setTextStyle(${style.index});");
  }

/**
 * @brief 获取文本样式
 * @return 文本样式
*/
  Future<TEduBoardTextStyle> getTextStyle() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getTextStyle();");
    if (result == null) {
      return null;
    }
    return TEduBoardTextStyle.values[int.parse(result)];
  }

/**
 * @brief 设置文本大小
 * @param size                 要设置的文本大小
 *
 * 实际像素值取值(size * 白板的高度 / 10000)px
*/
  void setTextSize(int size) {
    _evaluateJavaScript("this.board.setTextSize($size);");
  }

/**
 * @brief 获取文本大小
 * @return 文本大小
*/
  Future<int> getTextSize() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getTextSize();");
    if (result == null) {
      return null;
    }
    return int.parse(result);
  }

/**
 * @brief 设置直线样式
 * @param style             要设置的直线样式
*/
  void setLineStyle(TEduBoardLineStyle style) {
    _evaluateJavaScript(
        "this.board.setLineStyle({lineType:${style.lineType.index},startArrowType:${style.startArrowType.index},endArrowType:${style.endArrowType.index}});");
  }

/**
 * @brief 获取直线样式
 * @return 直线样式
*/
  Future<TEduBoardLineStyle> getLineStyle() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getLineStyle();");
    Map resultMap = jsonDecode(result);
    TEduBoardLineType lineType =
        TEduBoardLineType.values[resultMap['lineType']];
    TEduBoardArrowType startArrowType =
        TEduBoardArrowType.values[resultMap['startArrowType']];
    TEduBoardArrowType endArrowType =
        TEduBoardArrowType.values[resultMap['endArrowType']];
    TEduBoardLineStyle style = TEduBoardLineStyle(
        lineType: lineType,
        startArrowType: startArrowType,
        endArrowType: endArrowType);
    return style;
  }

/**
 * @brief 设置椭圆绘制模式
 * @param mode             要设置的椭圆绘制模式
*/
  void setOvalDrawMode(TEduBoardOvalDrawMode mode) {
    _evaluateJavaScript("this.board.setOvalDrawMode(${mode.index});");
  }

/**
 * @brief 获取椭圆绘制模式
 * @return 椭圆绘制模式
*/
  Future<TEduBoardOvalDrawMode> getOvalDrawMode() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getOvalDrawMode();");
    if (result == null) {
      return null;
    }
    return TEduBoardOvalDrawMode.values[int.parse(result)];
  }

/**
 * @brief 是否启用原生系统光标
 * @param enable 启用或禁用，默认禁用
*/
  void setSystemCursorEnable(bool enable) {
    _evaluateJavaScript("this.board.setSystemCursorEnable($enable);");
  }

/**
 * @brief 撤销当前白板页上一次动作
*/
  void undo() {
    _evaluateJavaScript("this.board.undo();");
  }

/**
 * @brief 重做当前白板页上一次撤销
*/
  void redo() {
    _evaluateJavaScript("this.board.redo();");
  }

/**
 * @brief 清除涂鸦，同时清空背景色以及背景图片
 */
  void clear() {
    _evaluateJavaScript("this.board.clear(true);");
  }

/**
 * @brief 清除涂鸦
 */
  void clearDraws() {
    _evaluateJavaScript("this.board.clear();");
  }

/**
 * @brief 清空当前白板页涂鸦
 * @param background     是否同时清空背景色以及背景图片
 * @param selected 是否只清除选中部分涂鸦
 * @warning 目前不支持清除选中部分的同时清除背景
*/
  void clearBackground(bool background, bool selected) {
    _evaluateJavaScript("this.board.clear($background, $selected);");
  }

/**
 * @brief 自定义白板工具鼠标样式
 * @param toolType              要设置鼠标样式的白板工具类型
 * @param cursorIcon          要设置的鼠标样式
*/
  void setCursorIcon(
      TEduBoardToolType toolType, TEduBoardCursorIcon cursorIcon) {
    _evaluateJavaScript(
        "this.board.setCursorIcon(${toolType.index}, {cursor:'${cursorIcon.cursor}', url:'${cursorIcon.url}', offsetX:${cursorIcon.offsetX}, offsetY:${cursorIcon.offsetY}});");
  }

/**
 * @brief 设置画笔和激光笔工具提示语
 * @param title         提示语
 * @param style          如果为空，则使用默认样式
*/
  void setToolTypeTitle(String title, {TEduBoardToolTypeTitleStyle style}) {
    if (style != null) {
      _evaluateJavaScript(
          "this.board.setToolTypeTitle('$title', {position: ${style.position.index}, size:${style.size}, color:'${rgbColor(style.color)}', style:${style.style.index});");
    } else {
      _evaluateJavaScript("this.board.setToolTypeTitle('$title');");
    }
  }

////白板页操作接口
/**
 * @brief 增加一页白板
 * @param url                要使用的背景图片 URL，编码格式为 UTF8，为 nullptr 表示不指定背景图片
 * @return 白板ID
 * @warning 白板页会被添加到默认文件（文件 ID 为#DEFAULT)，自行上传的文件无法添加白板页
 *
 * 返回值内存由SDK内部管理，用户不需要自己释放
 */
  Future<String> addBoardWithBackgroundImage(String url) async {
    String jsCode = "this.board.addBoard('');";
    if (url != null && url.length > 0) {
      jsCode = "this.board.addBoard('$url');";
    }
    String result = await _jsonDecodeEvaluateJavaScript(jsCode);
    return result;
  }

/**
 * @brief 删除一页白板
 * @param boardId            要删除的白板 ID，为 nullptr 表示删除当前页
 * @warning 只允许删除默认文件（文件 ID 为#DEFAULT）内的白板页，且默认白板页（白板 ID 为#DEFAULT）无法删除
*/
  void deleteBoard(String boardId) {
    _evaluateJavaScript("this.board.deleteBoard('$boardId');");
  }

/**
 * @brief 上一步
 *  每个 Step 对应 PPT 的一个动画效果，若当前没有已展示的动画效果，则该接口调用会导致向前翻页
*/
  void prevStep() {
    _evaluateJavaScript("this.board.prevStep();");
  }

/**
 * @brief 下一步
 *
 * 每个 Step 对应 PPT 的一个动画效果，若当前没有未展示的动画效果，则该接口调用会导致向后翻页
*/
  void nextStep() {
    _evaluateJavaScript("this.board.nextStep();");
  }

/**
 * @brief 向前翻页
 *
 * 若当前白板页为当前文件的第一页，则该接口调用无效
*/
  void preBoard() {
    _evaluateJavaScript("this.board.prevBoard();");
  }

/**
 * @brief 向后翻页
 *
 * 若当前白板页为当前文件的最后一页，则该接口调用无效
*/
  void nextBoard() {
    _evaluateJavaScript("this.board.nextBoard();");
  }

/**
 * @brief 跳转到指定白板页
 * @param boardId            要跳转到的白板页 ID
 *
 * 允许跳转到任意文件的白板页
*/
  void gotoBoard(String boardId) {
    _evaluateJavaScript("this.board.gotoBoard('$boardId');");
  }

/**
 * @brief 向前翻页
 * @param resetStep            指定翻到指定页以后是否重置 PPT 动画步数
 *
 * 若当前白板页为当前文件的第一页，则该接口调用无效
*/
  void preBoardAndReset(bool resetStep) {
    _evaluateJavaScript("this.board.prevBoard($resetStep);");
  }

/**
 * @brief 向后翻页
 * @param resetStep            指定翻到指定页以后是否重置 PPT 动画步数
 *
 * 若当前白板页为当前文件的最后一页，则该接口调用无效
*/
  void nextBoardAndReset(bool resetStep) {
    _evaluateJavaScript("this.board.nextBoard($resetStep);");
  }

/**
 * @brief 跳转到指定白板页
 * @param boardId            要跳转到的白板页 ID
 * @param resetStep        指定翻到指定页以后是否重置 PPT 动画步数
 *
 * 允许跳转到任意文件的白板页
*/
  void gotoBoardAndReset(String boardId, bool resetStep) {
    _evaluateJavaScript("this.board.gotoBoard('$boardId', $resetStep);");
  }

/**
 * @brief 获取当前白板页 ID
 * @return 当前白板页 ID
 *
 * 返回值内存由 SDK 内部管理，用户不需要自己释放
*/
  Future<String> getCurrentBoard() async {
    return _jsonDecodeEvaluateJavaScript("this.board.getCurrentBoard();");
  }

/**
 * @brief 获取所有文件的白板列表
 * @return 所有文件的白板列表
*/
  Future<List<String>> getBoardList() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getBoardList();");
    List<dynamic> list = json.decode(result);
    List<String> boardList = [];
    list.forEach((element) {boardList.add(element);});
    return boardList;
  }

/**
 * @brief 设置当前白板页宽高比
 * @param ratio                要设置的白板宽高比
 *
 * 格式如: "4:3"、"16:9"
*/
  void setBoardRatio(String ratio) {
    _evaluateJavaScript("this.board.setBoardRatio('$ratio');");
  }

/**
 * @brief 获取当前白板页宽高比
 * @return 白板宽高比，格式与 SetBoardRatio 接口参数格式一致
*/
  Future<String> getBoardRatio() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getBoardRatio();");
    return result;
  }

/**
 * @brief 设置当前白板页缩放比例
 * @param scale                要设置的白板缩放比例
 *
 * 支持范围: [100，1600]，实际缩放比为: scale / 100
*/
  void setBoardScale(int scale) {
    _evaluateJavaScript("this.board.setBoardScale($scale);");
  }

/**
 * @brief 获取当前白板页缩放比例
 * @return 白板缩放比例，格式与 SetBoardScale 接口参数格式一致
*/
  Future<int> getBoardScale() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getBoardScale();");
    if (result == null) {
      return null;
    }
    return int.parse(result);
  }

/**
 * @brief 设置白板内容自适应模式
 * @param mode                要设置的白板内容自适应模式
 *
 * 设置自适应模式后会影响所有后续白板内容操作,受影响接口包括：AddTranscodeFile
*/
  void setBoardContentFitMode(TEduBoardContentFitMode mode) {
    _evaluateJavaScript("this.board.setBoardContentFitMode(${mode.index});");
  }

/**
 * @brief 获取白板内容自适应模式
 * @return 白板内容自适应模式
*/
  Future<TEduBoardContentFitMode> getBoardContentFitMode() async {
    String result =
        await _jsonDecodeEvaluateJavaScript("this.board.getBoardContentFitMode();");
    if (result == null) {
      return null;
    }
    return TEduBoardContentFitMode.values[int.parse(result)];
  }

/**
 * 添加白板元素
 * @param   url       网页或者图片的 url，只支持 https 协议的网址或者图片 url
 * @param   type     元素类型，当设置TEDU_BOARD_ELEMENT_IMAGE时，等价于addImageElement方法
 * @return  元素ID
 * @warning
 * （1）当 type = TEDU_BOARD_ELEMENT_IMAGE，支持 png、jpg、gif、svg 格式的本地和网络图片，当 url 是一个有效的本地文件地址时，该文件会被自动上传到 COS，上传进度回调 onTEBFileUploadStatus
 * （2）当 type = TEDU_BOARD_ELEMENT_CUSTOM_GRAPH，仅支持网络 url，请与自定义图形工具 TEDU_BOARD_TOOL_TYPE_BOARD_CUSTOM_GRAPH 配合使用
 * （3）当 type = TEDU_BOARD_ELEMENT_AUDIO 或 TEDU_BOARD_ELEMENT_GLOBAL_AUDIO，仅支持网络 url
 */
  Future<String> addElement(String url, TEduBoardElementType type) async {
    String data = await fileData(url);

    if (data == null || data.length <= 0) {
      return _jsonDecodeEvaluateJavaScript(
          "this.board.addElement(${type.index}, '$url');");
    }

    //本地文件
    return _jsonDecodeEvaluateJavaScript("this.board.addElement(${type.index}, $data);");
  }

/**
 * 删除白板元素
 * @param elementId     元素 ID
 * @return 删除是否成功
 */
  Future<bool> removeElement(String elementId) async {
    String result = await _jsonDecodeEvaluateJavaScript(
        "this.board.removeElement('${elementId ?? ''}');");
    return toBoolean(result);
  }

/**
 * 预设文本工具内容
 * @param input     预设文本内容，取消预设则设置为空
 * @param focus     是否继续保持焦点
 */
  void setNextTextInput(String input, bool focus) {
    _evaluateJavaScript("this.board.setNextTextInput('$input', $focus);");
  }

/**
 * 预设文本工具内容
 * @param zoomIn     放大工具图标
 * @param zoomOut   缩小工具图标
 * @warning 该接口只在桌面端支持
 */
  void setZoomCursorIcon(
      TEduBoardCursorIcon zoomIn, TEduBoardCursorIcon zoomOut) {
    _evaluateJavaScript(
        "this.board.setZoomCursorIcon({cursor:'${zoomIn.cursor}', url:'${zoomIn.url}', offsetX:${zoomIn.offsetX}, offsetY:${zoomIn.offsetY}}, {cursor:'${zoomOut.cursor}', url:'${zoomOut.url}', offsetX:${zoomOut.offsetX}, offsetY:${zoomOut.offsetY}});");
  }

/**
 * @brief 设置白板是否开启笔锋
 * @param enable             【必填】是否开启，true 表示开启，false 表示关闭
 *
 * 白板创建后默认为关闭
*/
  void setHandwritingEnable(bool enable) {
    _evaluateJavaScript("this.board.setHandwritingEnable($enable);");
  }

/**
 * @brief 获取白板是否开启笔锋
 * @return 是否开启笔锋
*/
  Future<bool> isHandwritingEnable() async {
    String result =
        await _jsonDecodeEvaluateJavaScript("this.board.isHandwritingEnable();");
    return toBoolean(result);
  }

/**
 * @brief 刷新当前页白板，触发 onTEBRefresh 回调
 * @warning 如果当前白板包含 PPT/H5/图片/视频时，刷新白板将会触发对应的回调
*/
  void refresh() {
    _evaluateJavaScript("this.board.refresh();");
  }

/**
 * @brief 同步本地发送失败的数据到远端并刷新本地数据
 *
 * 接口用途：此接口主要用于网络恢复后，同步本地数据到远端，拉取远端数据到本地
 * 调用时机：在网络恢复后调用
 * 使用限制：
 * （1）仅支持2.4.9及以上版本
 * （2）如果历史数据还没有加载完成，则不允许重复调用，否则回调告警 TEDU_BOARD_WARNING_ILLEGAL_OPERATION
 * @warning Reload等同于重新加载历史数据，会触发白板初始化时除onTEBInit之外的所有回调。
*/
  void syncAndReload() {
    _evaluateJavaScript("this.board.syncAndReload();");
  }

  /**
  * @brief 白板快照
  * @param info 快照信息
 */
  void snapshot({TEduBoardSnapshotInfo info}) {
    if (info == null || info.path.length <= 0) {
      for (var item in _boardView?.operationCallbacksHandler) {
        item?.onTEBSnapshot(info.path, TEduBoardErrorCode.TEDU_BOARD_ERROR_PATH_INVALID, '');
      }
      return;
    }
    _evaluateJavaScript(
        "this.board.snapshot({userData:'${eduStringEscape(info.path)}'});");
  }

/**
 * @brief 设置缩放锚点
 * @param xRatio 距离白板左边位置，取值【0, 1】
 * @param yRatio 距离白板顶部位置，取值【0, 1】
*/
  void setScaleAnchor(double xRatio, double yRatio) {
    _evaluateJavaScript("this.board.setScaleAnchor($xRatio, $yRatio);");
  }

/**
 * @brief 设置远端画笔在本地是否可见
 * @param visible  是否可见
*/
  void setRemoteCursorVisible(bool visible) {
    _evaluateJavaScript("this.board.setRemoteCursorVisible($visible);");
  }

/**
 * @brief 设置缩放工具的缩放比例
 * @param scale  最小缩放步长
*/
  void setScaleToolRatio(int scale) {
    _evaluateJavaScript("this.board.setScaleToolRatio($scale);");
  }

////文件操作接口
/**
 * @brief 发起文件转码请求
 * @param path                要转码的文件路径，编码格式为 UTF8
 * @param config            转码参数
 * @warning 本接口设计用于在接入阶段快速体验转码功能，原则上不建议在生产环境中使用，生产环境中的转码请求建议使用后台服务接口发起
 * @see TEduBoardCallback::onTEBFileTranscodeProgress
 *
 * 支持 PPT、PDF、Word 文件转码
 * PPT 文档默认转为H5动画，能够还原 PPT 原有动画效果，其它文档转码为静态图片
 * PPT 动画转码耗时约1秒/页，所有文档的静态转码耗时约0.5秒/页
 * 转码进度和结果将会通过 onTEBFileTranscodeProgress 回调返回，详情参见该回调说明文档
 */
  Future<void> applyFileTranscode(
      String path, TEduBoardTranscodeConfig config) async {
    String data = await fileData(path);

    if (data == null || data.length <= 0) {
      TIWLog.logEvent("applyFileTranscode, path:$path, code:-1, desc:file not exists ");
      return;
    }

    String jsCode = "this.board.applyFileTranscode($data,{minResulution:'${config.minRosolution ?? ''}',isStaticPPT:${config.isStaticPPT},thumbnailResolution:'${config.thumbnailResolution ?? ''}'});";
    return _jsonDecodeEvaluateJavaScript(jsCode);
  }

/**
 * @brief 主动查询文件转码进度
 * @param taskId            通过 onTEBFileTranscodeProgress 回调拿到的转码任务 taskId
 * @warning 该接口仅用于特殊业务场景下主动查询文件转码进度，调用 ApplyFileTranscode 后，SDK 内部将会自动定期触发 onTEBFileTranscodeProgress 回调，正常情况下您不需要主动调用此接口
 * @see TEduBoardCallback::onTEBFileTranscodeProgress
 *
 * 转码进度和结果将会通过 onTEBFileTranscodeProgress 回调返回，详情参见该回调说明文档
*/
  void getFileTranscodeProgress(String taskId) {
    _evaluateJavaScript("this.board.getFileTranscodeProgress('$taskId');");
  }

/**
 * @brief 添加转码文件
 * @param  result             文件转码结果
 * @param  needSwitch    是否跳转到该文件
 * @return 文件ID
 * @warning 当传入文件的 URL 重复时，返回 URL 对应的 文件 ID
 * @warning 在收到对应的 onTEBAddTranscodeFile 回调前，无法用返回的文件 ID 查询到文件信息
 * @see TEduBoardCallback::onTEBAddTranscodeFile
 *
 * TEduBoardTranscodeFileResult 的字段信息主要来自：
 * 1. 使用客户端 ApplyFileTranscode 转码，直接将转码结果用于调用此接口
 * 2.  （推荐）使用服务端 REST API 转码，只需传入转码回调结果的四个字段（title，resolution，url，pages），其服务端->客户端字段的对应关系为 Title->title、Resolution->resolution、ResultUrl->url、Pages->pages [转码文档](https://cloud.tencent.com/document/product/1137/40260)
 *
 * 调用该接口后，SDK 会在后台进行文件加载，期间用户可正常进行其它操作，加载成功或失败后会触发相应回调
 * 文件加载成功后，将自动切换到该文件
*/
  Future<String> addTranscodeFile(
      TEduBoardTranscodeFileResult result, bool needSwitch) async {
    String fileId = await _jsonDecodeEvaluateJavaScript(
        "this.board.addTranscodeFile({title:'${eduStringEscape(result.title)}',resolution:'${result.resolution}',url:'${result.url}',pages:${result.pages}}, $needSwitch);");
    return fileId;
  }

/**
 * @brief 删除文件
 * @param fileId            要删除的文件 ID
 *
 * 文件 ID 为 nullptr 时表示当前文件，默认文件无法删除
*/
  void deleteFile(String fileId) {
    _evaluateJavaScript("this.board.deleteFile('$fileId');");
  }

/**
 * @brief 切换文件
 * @param fileId            要切换到的文件 ID
 * @warning 该接口仅可用于文件切换，如果传入的 fileId 为当前文件 ID，SDK 会忽略其它参数，不做任何操作
 *
 * 文件 ID 为必填项，为 nullptr 或空字符串将导致文件切换失败
*/
  void switchFile(String fileId) {
    _evaluateJavaScript("this.board.switchFile('$fileId');");
  }

/**
 * @brief 切换文件
 * @param fileId            要切换到的文件 ID
 * @param boardId           切换文件并跳转到这个白板页
 * @param stepIndex         跳转到白板页并切换到这个动画
 * @warning 该接口仅可用于文件切换，如果传入的 fileId 为当前文件 ID，SDK 会忽略其它参数，不做任何操作
 *
 * 文件 ID 为必填项，为 nullptr 或空字符串将导致文件切换失败
*/
  void switchFileWithBoardId(String fileId, String boardId, int stepIndex) {
    _evaluateJavaScript(
        "this.board.switchFile('$fileId','$boardId', $stepIndex);");
  }

/**
 * @brief 获取当前文件 ID
 * @return 当前文件 ID
*/
  Future<String> getCurrentFile() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getCurrentFile();");
    return result;
  }

/**
 * @brief 获取白板中指定文件的文件信息
 * @return 文件信息
 * @warning 每次调用该接口的返回值内容都指向同一块内存，若需要保存返回信息，请在拿到返回值后及时拷贝走
*/
  Future<TEduBoardFileInfo> getFileInfo(String fileId) async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getFileInfo('$fileId');");
    Map<String, dynamic> fileInfo = json.decode(result);
    List<dynamic> boardInfoList = fileInfo['boardInfoList'];
    List<TEduBoardInfo> boards = [];
    boardInfoList.forEach((boardInfo) {
      boards.add(TEduBoardInfo(boardInfo['boardId'], boardInfo['backgroundUrl'], hexColor(boardInfo['backgroundColor'])));
    });
    return TEduBoardFileInfo(
      fileInfo['fid'], 
      fileInfo['title'],
      fileInfo['downloadURL'],
      fileInfo['currentPageIndex'],
      fileInfo['pageCount'],
      boards
      );
  }

/**
 * @brief 获取白板中上传的所有文件的文件信息列表
 * @return 文件信息列表
 * @warning 返回值不再使用时不需要自行 delete，但是务必调用其 release 方法以释放内存占用
*/
  Future <List<TEduBoardFileInfo>> getFileInfoList() async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getFileInfoList();");
    List<dynamic> fileInfoList = json.decode(result);
    List<TEduBoardFileInfo> files = [];
    fileInfoList.forEach((fileInfo) {
      List<dynamic> boardInfoList = fileInfo['boardInfoList'];
      List<TEduBoardInfo> boards = [];
      boardInfoList.forEach((boardInfo) {
        boards.add(TEduBoardInfo(boardInfo['boardId'], boardInfo['backgroundUrl'], hexColor(boardInfo['backgroundColor'])));
      });
      TEduBoardFileInfo file = TEduBoardFileInfo(
        fileInfo['fid'], 
        fileInfo['title'],
        fileInfo['downloadURL'],
        fileInfo['currentPageIndex'],
        fileInfo['pageCount'],
        boards
      );
      files.add(file);
    });
    return files;
  }

/**
 * @brief 获取指定文件的白板 ID 列表
 * @param fileId            文件 ID
 * @return 白板 ID 列表
 * @warning 返回值不再使用时不需要自行 delete，但是务必调用其 release 方法以释放内存占用
*/
  Future <List<String>>  getFileBoardList(String fileId) async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getFileBoardList('$fileId');");
    List<dynamic> list = json.decode(result);
    List<String> boardList = [];
    list.forEach((element) { boardList.add(element); });
    return boardList;
  }

/**
 * @brief 获取指定文件的缩略图，不支持默认文件（fileId=#DEFAULT）
 * @param fileId            文件 ID
 * @return 缩略图 URL 列表
 *
 * 用户在调用 rest api 请求转码时，需要带上 "thumbnail_resolution" 参数，开启缩略图功能，否则返回的缩略图 url 无效
*/
  Future<List<String>> getThumbnailImages(String fileId) async {
    String result = await _jsonDecodeEvaluateJavaScript("this.board.getThumbnailImages('$fileId');");
    List<dynamic> list = json.decode(result);
    List<String> imageList = [];
    list.forEach((element) { imageList.add(element); });
    return imageList;
  }

/**
 * @brief 获取白板中所有元素
 * @param boardId   白板 ID，如果为空则获取当前白板所有元素
 * @return 白板元素列表
*/
  Future<List<TEduBoardElementInfo>> getBoardElementList(String boardId) async {
    String value = await _jsonDecodeEvaluateJavaScript(
        "this.board.getBoardElementList('${boardId ?? ''}');");
    List elements = jsonDecode(value);
    List<TEduBoardElementInfo> result = [];
    for (var item in elements) {
      TEduBoardElementInfo info = TEduBoardElementInfo(
          boardId: item['boardId'],
          elementId: item['elementId'],
          type: TEduBoardElementType.values[item['type']],
          url: item['url']);
      result.add(info);
    }
    return result;
  }

/**
 * @brief 清空指定文件的所有白板涂鸦
 * @param fileId            文件 ID
*/
  void clearFileDraws(String fileId) {
    _evaluateJavaScript("this.board.clearFileDraws(['$fileId']);");
  }

/**
 * @brief 添加视频文件
 * @param url         文件播放地址
 * @return 文件 ID
 * @warning 当传入文件的 URL 重复时，返回 URL 对应的 文件 ID
 *
 * 支持 mp4/m3u8/hls，触发状态改变回调 onTEBVideoStatusChanged
 */
  Future<String> addVideoFile(String url) async {
    String fileId =
        await _jsonDecodeEvaluateJavaScript("this.board.addVideoFile('$url');");
    return fileId;
  }

/**
 * @brief 显示或隐藏视频控制栏
 * @param show        是否显示
 * @warning 全局控制项，对所有视频文件有效
 * 隐藏和显示默认视频控制栏，默认显示系统自带的 video 控制栏，不同平台界面 UI 样式不同
 */
  void showVideoControl(bool show) {
    _evaluateJavaScript("this.board.showVideoControl($show);");
  }

/**
 * @brief 播放视频
 * @warning 只对当前文件有效
 *
 * 触发状态改变回调 onTEBVideoStatusChange，一般在使用自定义视频控制栏时使用
 * 移动端回前台调用 play（WebView 默认行为）
 */
  void playVideo() {
    _evaluateJavaScript("this.board.playVideo();");
  }

/**
 * @brief 暂停视频
 * @warning 只对当前文件有效
 *
 * 触发状态改变回调 onTEBVideoStatusChanged，一般在使用自定义视频控制栏时使用
 * 移动端退后台调用 pause（WebView 默认行为）
*/
  void pauseVideo() {
    _evaluateJavaScript("this.board.pauseVideo();");
  }

/**
 * @brief 跳转（仅支持点播视频）
 * @param time 播放进度，单位秒
 * @warning 只对当前文件有效
 *
 * 触发状态改变回调 onTEBVideoStatusChanged，一般在使用自定义视频控制栏时使用
 */
  void seekVideo(double time) {
    _evaluateJavaScript("this.board.seekVideo($time);");
  }

/**
 * @brief 是否同步本地视频操作到远端
 * @param enable            【必填】是否同步
 * @warning 全局控制项，对所有视频文件有效
 *
 * play/pause/seek 接口以及控制栏事件的触发是否影响远端，默认为 true
 * 一般情况下学生设置为 false，老师设置为 true
 */
  void setSyncVideoStatusEnable(bool enable) {
    _evaluateJavaScript("this.board.setSyncVideoStatusEnable($enable);");
  }

/**
 * @brief 内部启动定时器，定时同步视频状态到远端（仅限于 mp4）
 * @param interval            【选填】同步间隔，例如设置5秒
 * @warning 只对当前文件有效
 *
 * 一般在老师端视频加载完成后调用，切换文件后内部自动销毁定时器，
 */
  void startSyncVideoStatus(int interval) {
    _evaluateJavaScript("this.board.startSyncVideoStatus($interval)");
  }

/**
 * @brief 停止同步视频状态
 * @warning 只对当前文件有效
 */
  void stopSyncVideoStatus() {
    _evaluateJavaScript("this.board.stopSyncVideoStatus();");
  }

/**
 * @brief 是否同步本地音频操作到远端
 * @param enable            【必填】是否同步
 * @warning 全局控制项，对所有视频文件有效
 *
 * play/pause/seek 接口以及控制栏事件的触发是否影响远端，默认为 true
 * 一般情况下学生设置为 false，老师设置为 true
 */
  void setSyncAudioStatusEnable(bool enable) {
    _evaluateJavaScript(
        "this.board.setSyncAudioStatusEnable(${enable ? 'true' : 'false'});");
  }

/**
 * @brief 是否启用音频控制面板
 * @param enable        启用或禁止
 * @warning 禁止控制面板后，不能通过界面交互方式操作音频元素
 */
  void enableAudioControl(bool enable) {
    _evaluateJavaScript(
        "this.board.enableAudioControl(${enable ? 'true' : 'false'});");
  }

/**
 * @brief 播放音频
 * @param elementId     调用 addElement 方法返回的元素 ID
 *
 * 触发状态改变回调 onTEBAudioStatusChange
 */
  void playAudio(String elementId) {
    _evaluateJavaScript("this.board.playAudio('$elementId');");
  }

/**
 * @brief 暂停音频
 * @param elementId     调用 addElement 方法返回的元素 ID
 *
 * 触发状态改变回调 onTEBAudioStatusChange
 */
  void pauseAudio(String elementId) {
    _evaluateJavaScript("this.board.pauseAudio('$elementId');");
  }

/**
 * @brief 跳转
 * @param elementId     调用 addElement 方法返回的元素 ID
 * @param time                播放进度，单位秒
 *
 * 触发状态改变回调 onTEBAudioStatusChange
 */
  void seekAudio(String elementId, double time) {
    _evaluateJavaScript("this.board.seekAudio('$elementId', ${time.toInt()});");
  }

/**
 * @brief 设置音量
 * @param elementId     调用 addElement 方法返回的元素 ID
 * @param volume  音频音量，取值范围[0-1]
 */
  void setAudioVolume(String elementId, double volume) {
    _evaluateJavaScript("this.board.setAudioVolume('$elementId', $volume);");
  }

  /**
  * 获取音量
  * @param elementId     调用 addElement 方法返回的元素 ID
  * @return 当前音量，取值范围[0-1]
  */
  Future<double> getAudioVolume(String elementId) async {
    String volume =
        await _jsonDecodeEvaluateJavaScript("this.board.getAudioVolume('$elementId');");
    if (volume == null) {
      return null;
    }
    return double.parse(volume);
  }

/**
 * @brief 添加 H5 页面
 * @param url            【必填】网页地址
 * @return 文件 ID
 * @warning 只支持展示，不支持互动
 */
  Future<String> addH5File(String url) async {
    String fileId = await _jsonDecodeEvaluateJavaScript("this.board.addH5File('$url');");
    return fileId;
  }

/**
 * @brief 批量导入图片到白板
 * @param urls               要使用的背景图片 URL 列表，编码格式为 UTF8
 * @return 新增加文件Id
 * @warning 当传入文件的 URL 重复时，返回 URL 对应的 文件 ID
 */
  Future<String> addImagesFile(List<String> urls) async {
    if (urls == null || urls.length <= 0) {
      return null;
    }
    String param = jsonEncode(urls);
    String fileId =
        await _jsonDecodeEvaluateJavaScript("this.board.addImagesFile($param);");
    return fileId;
  }

  void setTextValue(String elementId, String textData) {
    print("InputDialog setTextValue elementId: $elementId, textData: $textData");
    _evaluateJavaScript("this.board.setTextValue('$elementId', {value: '$textData'});");
  }
}
