import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'package:flutter/material.dart';
import 'package:tiw_flutter/teduboad_callbackhandler.dart';
import 'package:tiw_flutter/tiw_log.dart';
import 'InputDialog.dart';
import 'teduboard_def.dart';
import 'teduboard_controller.dart';
import 'package:webview_flutter/webview_flutter.dart';

typedef void TeduBoardViewInitCallback(TEduBoardController boardController);

const String TEDU_BOARD_HTML_ANDROID =
    'http://down.qq.com/innovate/qqclass/TEduBoard_Android.html';

const String TEDU_BOARD_HTML =
    'https://res.qcloudtiw.com/board/2.5.7/TEduBoard_Mac_iOS.html?t=';

//log前缀
const String  LOG_TAG = '[TeduBoardView]';

// js回调的log前缀
const String  JSCB_LOG_TAG = '$LOG_TAG board callback';

// ignore: must_be_immutable
class TeduBoardView extends StatefulWidget {
  ///授权参数
  final TEduBoardAuthParam authParam;

  ///课堂 ID，32位整型，取值范围[1, 4294967294]
  final int roomId;

  ///可选参数，指定用于初始化白板的一系列属性值
  ///@warning 使用腾讯云 IMSDK 进行实时数据同步时，只支持一个白板实例，创建多个白板实例可能导致涂鸦状态异常
  final TEduBoardInitParam initParam;

  ///通用事件回调
  Set<TeduBoardCommonCallbacksHandler> commonCallbacksHandler = new Set();

  ///文件操作回调
  Set<TeduBoardFileOperationCallbacksHandler> fileOperationHandler = new Set();

  ///涂鸦操作回调
  Set<TeduBoardGraffitiCallbacksHandler> graffitiCallbacksHandler = new Set();

  ///白板操作回调
  Set<TeduBoardOperationCallbacksHandler> operationCallbacksHandler = new Set();

  ///白板基本流程回调
  Set<TeduBoardProcessCallbacksHandler> processCallbacksHandler = new Set();

  TeduBoardView(
      {Key key,
      @required this.authParam,
      @required this.roomId,
      this.initParam})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _TeduBoardViewState();
  }

  ///通用事件回调
  void addCommonCallbacksHandler(
      TeduBoardCommonCallbacksHandler handler) {
    if (handler == null) {
      return;
    }
    commonCallbacksHandler.add(handler);
  }

  ///基本流程回调
  void addProcessCallbacksHandler(TeduBoardProcessCallbacksHandler handler) {
    if (handler == null) {
      return;
    }
    processCallbacksHandler.add(handler);
  } 

  ///涂鸦操作回调
  void addGraffitiCallbacksHandler(TeduBoardGraffitiCallbacksHandler handler) {
    if (handler == null) {
      return;
    }
    graffitiCallbacksHandler.add(handler);
  }

  ///白板操作回调
  void addOperationCallbacksHandler(
      TeduBoardOperationCallbacksHandler handler) {
    if (handler == null) {
      return;
    }
    operationCallbacksHandler.add(handler);
  }

  ///文件操作回调
  void addFileOperationCallbacksHandler(
      TeduBoardFileOperationCallbacksHandler handler) {
    if (handler == null) {
      return;
    }
    fileOperationHandler.add(handler);
  }
}


class _TeduBoardViewState extends State<TeduBoardView> {
  TEduBoardController _boardController;

  @override
  void initState() {
    super.initState();
    if(Platform.isAndroid && widget.initParam.androidPlatformViewMod == AndroidPlatformViewMod.HYBRID_COMPOSITION) {
      WebView.platform = SurfaceAndroidWebView();
    }
  }

  @override
  Widget build(BuildContext context) {
    return WebView(
      initialUrl: _getBoardUrl(),
      javascriptMode: JavascriptMode.unrestricted,
      onWebViewCreated: (WebViewController webViewController) {
        _boardController = TEduBoardController(widget, webViewController);
      },
      onPageFinished: (String url) {
        initBoard(url);
      },
      javascriptChannels: <JavascriptChannel>[
        _onBoardCallbackChannel(context),
      ].toSet(),
      initialMediaPlaybackPolicy: AutoMediaPlaybackPolicy.always_allow,
      // dataBaseEnabled: true,
      // appCacheEnabled: true,
      debuggingEnabled: true,
    );
  }

  void initBoard(String url) {
    if (Platform.isAndroid && url.contains("innovate/qqclass/TEduBoard_Android.html")) {
      _boardController.initBoard();
    } else if (url.startsWith(_getBoardUrl())) {
      _boardController.initBoard();
    }
  }

  String _getBoardUrl() {
    return Platform.isAndroid ? TEDU_BOARD_HTML_ANDROID : TEDU_BOARD_HTML;
  }

  JavascriptChannel _onBoardCallbackChannel(BuildContext context) {
  return JavascriptChannel(
      name: 'onBoardCallback',
      onMessageReceived: (JavascriptMessage message) {

        Map<String, dynamic> param = json.decode(message.message);
        String event = param['event'];
      
        // 通用事件回调 (common)
        if (event == 'JS_ERROR') {
          _onJSError(param);
        }
        else if (event == 'TEB_ERROR') {
          _onBoardError(param);
        }
        else if (event == 'TEB_WARNING') {
          _onBoardWarning(param);
        }
        else if (event == 'TEB_INIT') {
          _onBoardInit(param);
        }
        // 白板基本流程回调 (process)
        else if (event == 'TEB_HISTROYDATA_SYNCCOMPLETED') {
          _onBoardHistroyDataSyncCompleted(param);
        }
        else if (event == 'TEB_SYNCDATA') {
          _onBoardSyncData(param);
        }
        else if (event == 'TEB_OPERATE_CANUNDO_STATUS_CHANGED') {
          _onBoardUndoStatusChanged(param);
        }
        else if (event == 'TEB_OPERATE_CANREDO_STATUS_CHANGED') {
          _onBoardRedoStatusChanged(param);
        }
        // 白板操作回调 (board operation) 
        else if (event == 'TEB_ADDBOARD') {
          _onAddBoard(param);
        }
        else if (event == 'TEB_DELETEBOARD') {
          _onDeleteBoard(param);
        }
        else if (event == 'TEB_GOTOBOARD') {
          _onGotoBoard(param);
        }
        else if (event == 'TEB_GOTOSTEP') {
          _onGotoStep(param);
        }
        else if (event == 'TEB_RECTSELECTED') {
          _onRectSelected(param);
        }
        else if (event == 'TEB_REFRESH') {
          _onRefresh(param);
        }
        else if (event == 'TEB_SNAPSHOT') {
          _onSnapshot(param);
        }
        /// 白板涂鸦功能回调 (graffiti)
        else if (event == 'TEB_IMAGE_STATUS_CHANGED') {
          _onTEBImageStatusChanged(param);
        }
        else if (event == 'TEB_SETBACKGROUNDIMAGE') {
          _onSetBackgroundImage(param);
        }
        else if (event == 'TEB_ADDELEMENT') {
          _onAddElement(param);
        }
        else if (event == 'TEB_REMOVEELEMENT') {
          _onRemoveElement(param);
        }
        else if (event == 'TEB_H5BACKGROUND_STATUS_CHANGED') {
          _onBoardBackgroundH5StatusChanged(param);
        }
        /// 文件操作回调
        else if (event == 'TEB_TRANSCODEPROGRESS') {
          _onTransCodeProcess(param);
        }
        else if (event == 'TEB_ADDTRANSCODEFILE') {
          _onAddTranscodeFile(param);
        }
        else if (event == 'TEB_DELETEFILE') {
          _onDeleteFile(param);
        }
        else if (event == 'TEB_SWITCHFILE') {
          _onSwitchFile(param);
        }
        else if (event == 'TEB_FILEUPLOADPROGRESS') {
          _onFileUploadProgress(param);
        }
        else if (event == 'TEB_FILEUPLOADSTATUS') {
          _onFileUploadStatus(param);
        }
        else if (event == 'TEB_H5FILE_STATUS_CHANGED') {
          _onH5FileStatusChanged(param);
        }
        else if (event == 'TEB_VIDEO_STATUS_CHANGED') {
          _onVideoStatusChanged(param);
        }
        else if (event == 'TEB_AUDIO_STATUS_CHANGED') {
          _onAudioStatusChanged(param);
        }
        else if (event == 'TEB_ADDIMAGESFILE') {
          _onAddImagesFile(param);
        }
        else if (event == 'TEB_H5PPT_STATUS_CHANGED') {
          _onH5PPTStatusChanged(param);
        }
        else if (event == 'TEB_TEXT_COMPONENT_STATUS_CHANGE') {
          print("_onBoardCallbackChannel TEB_TEXT_COMPONENT_STATUS_CHANGE");
          _onTextComponentStatusChange(context, param['data']);
        }
        else {
          TIWLog.logEvent('$LOG_TAG unhandled board callback message: ${message.message}');
        }
      });

  }

  // JS METHOD (common)

  void _onJSError(Map<String, dynamic> param) {
    TIWLog.logEvent('$JSCB_LOG_TAG onJSError: $param');
  }

  void _onBoardError(Map<String, dynamic> param) {
    int codeValue = param['code'];
    TEduBoardErrorCode errorCode = codeValue < TEduBoardErrorCode.values.length
        ? TEduBoardErrorCode.values[codeValue]
        : TEduBoardErrorCode.TEDU_BOARD_ERROR_NONE;
    String msg = param['msg'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBError, code: $errorCode, msg: $msg');
    for (var item in widget?.commonCallbacksHandler) {
      item?.onTEBError(errorCode, msg);
    }
  }

  void _onBoardWarning(Map<String, dynamic> param) {
    int codeValue = param['code'];
    TEduBoardWarningCode warningCode =
        codeValue < TEduBoardWarningCode.values.length
            ? TEduBoardWarningCode.values[codeValue]
            : TEduBoardWarningCode.TEDU_BOARD_WARNING_NONE;
    String msg = param['msg'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBWarning, code: $warningCode, msg: $msg');
    for (var item in widget?.commonCallbacksHandler) {
      item?.onTEBWarning(warningCode, msg);
    }
  }

  // JS METHOD (process)

  void _onBoardInit(Map<String, dynamic> param) {
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBInit');
    for (var item in widget?.processCallbacksHandler) {
      item?.onTEBInit(_boardController);
    }
  }

  void _onBoardHistroyDataSyncCompleted(Map<String, dynamic> param) {
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBHistroyDataSyncCompleted');
    for (var item in widget?.processCallbacksHandler) {
      item?.onTEBHistroyDataSyncCompleted();
    }
  }

  void _onBoardSyncData(Map<String, dynamic> param) {
    Map data = param['data'];
    String dataStr = json.encode(data);
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBSyncData, data: $dataStr');
    for (var item in widget?.processCallbacksHandler) {
      item?.onTEBSyncData(dataStr);
    }
  }

  void _onBoardUndoStatusChanged(Map<String, dynamic> param) {
    bool enable = param['enable'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBUndoStatusChanged, canRedo: $enable');
    for (var item in widget?.processCallbacksHandler) {
      item?.onTEBUndoStatusChanged(enable);
    }
  }

  void _onBoardRedoStatusChanged(Map<String, dynamic> param) {
    bool enable = param['enable'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBUndoStatusChanged, canRedo: $enable');
    for (var item in widget?.processCallbacksHandler) {
      item?.onTEBRedoStatusChanged(enable);
    }
  }

  // JS METHOD (board operation)

  void _onAddBoard(Map<String, dynamic> param) {
    List boadIds = param['boardIds'];
    String fileId = param['fileId'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBAddBoard, boardIds: $boadIds, fileId: $fileId');
    for (var item in widget?.operationCallbacksHandler) {
      item?.onTEBAddBoard(boadIds, fileId);
    }
  }

  void _onDeleteBoard(Map<String, dynamic> param) {
    List boadIds = param['boardIds'];
    String fileId = param['fileId'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBDeleteBoard, boardIds: $boadIds, fileId: $fileId');
    for (var item in widget?.operationCallbacksHandler) {
      item?.onTEBDeleteBoard(boadIds, fileId);
    }
  }

  void _onGotoBoard(Map<String, dynamic> param) {
    String boardId = param['boardId'];
    String fileId = param['fileId'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBGotoBoard, boardIds: $boardId, fileId: $fileId');
    for (var item in widget?.operationCallbacksHandler) {
      item?.onTEBGotoBoard(boardId, fileId);
    }
  }

  void _onGotoStep(Map<String, dynamic> param) {
    int currentStep = param['currentStep'];
    int totalStep = param['totalStep'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBGotoStep, currentStep: $currentStep, totalStep: $totalStep');
    for (var item in widget?.operationCallbacksHandler) {
      item?.onTEBGotoStep(currentStep, totalStep);
    }
  }

  void _onRectSelected(Map<String, dynamic> param) {
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBRectSelected');
    for (var item in widget?.operationCallbacksHandler) {
      item?.onTEBRectSelected();
    }
  }

  void _onRefresh(Map<String, dynamic> param) {
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBRefresh');
    for (var item in widget?.operationCallbacksHandler) {
      item?.onTEBRefresh();
    }
  }

  Future<void> _onSnapshot(Map<String, dynamic> param) async {
    Map data = param['data'];
    String base64Str = data['image'].replaceAll('data:image/png;base64,', '');
    String path = data['userData'];
    Uint8List bytes = base64.decode(base64Str);
    File file = File(path);
    try {
      await file.writeAsBytes(bytes);
      TIWLog.logEvent('$JSCB_LOG_TAG onTEBSnapshot, path: $path, code: ${TEduBoardErrorCode.TEDU_BOARD_ERROR_NONE}, msg: ');
      for (var item in widget?.operationCallbacksHandler) {
        item?.onTEBSnapshot(
            path, TEduBoardErrorCode.TEDU_BOARD_ERROR_NONE, null);
      }
    } catch (exception) {
      TIWLog.logWithError(exception, '$JSCB_LOG_TAG onTEBSnapshot');
      TIWLog.logEvent('$JSCB_LOG_TAG onTEBSnapshot, path: $path, code: ${TEduBoardErrorCode.TEDU_BOARD_ERROR_WRITE_ERROR}, msg: ${exception.toString()}');
      for (var item in widget?.operationCallbacksHandler) {
        item?.onTEBSnapshot(
            path, TEduBoardErrorCode.TEDU_BOARD_ERROR_WRITE_ERROR, exception.toString());
      }
    }
  }

  // JS METHOD (graffiti)

  void _onTEBImageStatusChanged(Map<String, dynamic> param) {
    int code = param['code'];
    TEduBoardImageStatus status = code < TEduBoardImageStatus.values.length
        ? TEduBoardImageStatus.values[code]
        : TEduBoardImageStatus.TEDU_BOARD_IMAGE_STATUS_PLACEHOLDER;
    Map data = param['data'];
    String boardId = data['currentBoardId'];
    String url = data['imgUrl'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBImageStatusChanged, boardId: $boardId, url: $url, status: $status');
    for (var item in widget?.graffitiCallbacksHandler) {
      item?.onTEBImageStatusChanged(boardId, url, status);
    }
  }

  void _onSetBackgroundImage(Map<String, dynamic> param) {
    String path = param['userData'];
    if (path == null || path == '') {
      path = param['fileUrl'];
    }
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBSetBackgroundImage, url: $path');
    for (var item in widget?.graffitiCallbacksHandler) {
      item?.onTEBSetBackgroundImage(path);
    }
  }

  void _onAddElement(Map<String, dynamic> param) {
    String path = param['userData'];
    String elementId = param['id'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBAddElement, elementId: $elementId, url: $path');
    for (var item in widget?.graffitiCallbacksHandler) {
      item?.onTEBAddElement(elementId, path);
    }
  }

  void _onRemoveElement(Map<String, dynamic> param) {
    List elementIds = param['ids'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBRemoveElement, elementIds: $elementIds');
    for (var item in widget?.graffitiCallbacksHandler) {
      item?.onTEBRemoveElement(elementIds);
    }
  }

  void _onBoardBackgroundH5StatusChanged(Map<String, dynamic> param) {
    int codeValue = param['code'];
    TEduBoardBackgroundH5Status status =
        codeValue < TEduBoardBackgroundH5Status.values.length
            ? TEduBoardBackgroundH5Status.values[codeValue]
            : TEduBoardBackgroundH5Status
                .TEDU_BOARD_BACKGROUND_H5_STATUS_PLACEHOLDER;
    Map data = param['data'];
    String boardId = data['currentBoardId'];
    String url = data['url'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBBackgroundH5StatusChanged, boardId: $boardId, url: $url, status: $status');
    for (var item in widget?.graffitiCallbacksHandler) {
      item?.onTEBBackgroundH5StatusChanged(boardId, url, status);
    }
  }

  // JS METHOD (file)
  void _onTransCodeProcess(Map<String, dynamic> param) {
    Map data = param['data'];
    String code = data['code'];
    String msg = data['message'];
    String path = data['userData'];
    String statusStr = data['status'];
    TEduBoardFileTranscodeStatus status =
        TEduBoardFileTranscodeStatus.TEDU_BOARD_FILE_TRANSCODE_ERROR; //默认是error
    if (statusStr == 'ERROR') {
      status = TEduBoardFileTranscodeStatus.TEDU_BOARD_FILE_TRANSCODE_ERROR;
    } else if (statusStr == 'UPLOADING') {
      status = TEduBoardFileTranscodeStatus.TEDU_BOARD_FILE_TRANSCODE_UPLOADING;
    } else if (statusStr == 'CREATED') {
      status = TEduBoardFileTranscodeStatus.TEDU_BOARD_FILE_TRANSCODE_CREATED;
    } else if (statusStr == 'QUEUED') {
      status = TEduBoardFileTranscodeStatus.TEDU_BOARD_FILE_TRANSCODE_QUEUED;
    } else if (statusStr == 'PROCESSING') {
      status =
          TEduBoardFileTranscodeStatus.TEDU_BOARD_FILE_TRANSCODE_PROCESSING;
    } else if (statusStr == 'FINISHED') {
      status = TEduBoardFileTranscodeStatus.TEDU_BOARD_FILE_TRANSCODE_FINISHED;
    }
    TEduBoardTranscodeFileResult result = TEduBoardTranscodeFileResult(
        data['taskId'],
        status,
        data['progress'],
        data['title'],
        data['resolution'],
        data['pages'],
        data['resultUrl'],
        data['thumbnailResolution'],
        data['thumbnailUrl']);
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBFileTranscodeProgress, result: { taskID: ${result.taskId}, status: ${result.status}, progress: ${result.progress}, title: ${result.title}, resolution: ${result.resolution}, pages: ${result.pages}, url: ${result.url}, thumbnailResolution: ${result.thumbnailResolution}, thumbnailUrl:${result.thumbnailUrl} }, path: $path, errorCode: $code, errorMsg: $msg');
    for (var item in widget?.fileOperationHandler) {
      item?.onTEBFileTranscodeProgress(result, path, code, msg);
    }
  }

  void _onAddTranscodeFile(Map<String, dynamic> param) {
    String fileId = param['fileId'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBAddTranscodeFile, fileId: $fileId');
    for (var item in widget?.fileOperationHandler) {
      item?.onTEBAddTranscodeFile(fileId);
    }
  }

  void _onDeleteFile(Map<String, dynamic> param) {
    String fileId = param['fileId'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBDeleteFile, fileId: $fileId');
    for (var item in widget?.fileOperationHandler) {
      item?.onTEBDeleteFile(fileId);
    }
  }

  void _onSwitchFile(Map<String, dynamic> param) {
    String fileId = param['fileId'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBSwitchFile, fileId: $fileId');
    for (var item in widget?.fileOperationHandler) {
      item?.onTEBSwitchFile(fileId);
    }
  }

  void _onFileUploadProgress(Map<String, dynamic> param) {
    Map data = param['data'];
    String path = data['userData'];
    int loaded = data['loaded'];
    int total = data['total'];
    num speed = data['speed'];
    num percent = data['percent'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBFileUploadProgress, path: $path, currentBytes: $loaded, totalBytes: $total, uploadSpeed: $speed, percent: $percent');
    for (var item in widget?.fileOperationHandler) {
      item?.onTEBFileUploadProgress(path, loaded, total, speed, percent);
    }
  }

  void _onFileUploadStatus(Map<String, dynamic> param) {
    int statusValue = param['status'];
    TEduBoardUploadStatus status =
        statusValue < TEduBoardUploadStatus.values.length
            ? TEduBoardUploadStatus.values[statusValue]
            : TEduBoardUploadStatus.TEDU_BOARD_UPLOAD_STATUS_PLACEHOLDER;
    Map data = param['data'];
    String path = data['userData'];
    String msg = data['errorMsg'];
    int code = data['code'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBFileUploadStatus, path: $path, status: $status, errorCode: $code, errorMsg: $msg');
    for (var item in widget?.fileOperationHandler) {
      item?.onTEBFileUploadStatus(path, status, code, msg);
    }
  }

  void _onH5FileStatusChanged(Map<String, dynamic> param) {
    Map data = param['data'];
    String fileId = data['fileId'];
    int statusValue = data['status'];
    TEduBoardH5FileStatus status =
        statusValue < TEduBoardH5FileStatus.values.length
            ? TEduBoardH5FileStatus.values[statusValue]
            : TEduBoardH5FileStatus.TEDU_BOARD_H5_FILE_STATUS_PLACEHOLDER;
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBH5FileStatusChanged, fileId: $fileId, status: $status');
    for (var item in widget?.fileOperationHandler) {
      item?.onTEBH5FileStatusChanged(fileId, status);
    }
  }

  void _onVideoStatusChanged(Map<String, dynamic> param) {
    Map data = param['data'];
    String fileId = data['fileId'];
    int statusValue = data['status'];
    TEduBoardVideoStatus status =
        statusValue < TEduBoardVideoStatus.values.length
            ? TEduBoardVideoStatus.values[statusValue]
            : TEduBoardVideoStatus.TEDU_BOARD_VIDEO_STATUS_PLACEHOLDER;
    num progress = data['progress'];
    num duration = data['duration'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBVideoStatusChanged, fileId: $fileId, status: $status, progress: $progress, duration: $duration');
    for (var item in widget?.fileOperationHandler) {
      item?.onTEBVideoStatusChanged(fileId, status, progress, duration);
    }
  }

  void _onAudioStatusChanged(Map<String, dynamic> param) {
    Map data = param['data'];
    String elementId = data['id'];
    int statusValue = data['status'];
    TEduBoardAudioStatus status =
        statusValue < TEduBoardAudioStatus.values.length
            ? TEduBoardAudioStatus.values[statusValue]
            : TEduBoardAudioStatus.TEDU_BOARD_AUDIO_STATUS_PLACEHOLDER;
    num progress = data['progress'];
    num duration = data['duration'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBAudioStatusChanged, elementId: $elementId, status: $status, progress: $progress, duration: $duration');
    for (var item in widget?.fileOperationHandler) {
      item?.onTEBAudioStatusChanged(elementId, status, progress, duration);
    }
  }

  void _onAddImagesFile(Map<String, dynamic> param) {
    String fileId = param['fileId'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBAddImagesFile, fileId: $fileId');
    for (var item in widget?.fileOperationHandler) {
      item?.onTEBAddImagesFile(fileId);
    }
  }

  void _onH5PPTStatusChanged(Map<String, dynamic> param) {
    Map data = param['data'];
    String fileId = data['fileId'];
    int statusValue = data['status'];
    TEduBoardH5PPTStatus status =
        statusValue < TEduBoardH5PPTStatus.values.length
            ? TEduBoardH5PPTStatus.values[statusValue]
            : TEduBoardH5PPTStatus.TEDU_BOARD_H5_PPT_STATUS_PLACEHOLDER;
    String message = data['message'];
    TIWLog.logEvent('$JSCB_LOG_TAG onTEBH5PPTStatusChanged, fileId: $fileId, status: $status, message: $message');
    for (var item in widget?.fileOperationHandler) {
      item?.onTEBH5PPTStatusChanged(fileId, status, message);
    }
  }

  /// 文本框组件事件通知
  void _onTextComponentStatusChange(BuildContext context, Map<String, dynamic> param) {
    TIWLog.logEvent("$JSCB_LOG_TAG onTextComponentStatusChange, status: $param['status'], elementId: $param['id'], value: $param['value']");

    // 只处理获取焦点事件，释放焦点事件不处理
    if (param['status'] == "blur") {
      return;
    }
    
    TextEditingController controller = TextEditingController();
    controller.text = param['value'];
    controller.selection = TextSelection.fromPosition(TextPosition(offset: controller.text.length));

    showDialog<void>(
      context: context,
      barrierColor: Color(0x8A000000),
      builder: (context) {
        return InputDialog(
            backgroundColor: Color(0x00000000),
            onSubmit: (inputText) => {
              _boardController.setTextValue(param['id'], inputText)
            },
            textController: controller,
        );
      },
    );
  }
}
